/*!
	@file
	@author		Albert Semenov
	@date		07/2012
*/

#ifndef _396f4ab3_a6ef_4ffc_91b8_40e335de896a_
#define _396f4ab3_a6ef_4ffc_91b8_40e335de896a_

namespace tools
{

	class MYGUI_EXPORT_DLL ComponentFactory
	{
	public:
		static void Initialise();
		static void Shutdown();
	};

}

#endif
