/*!
	@file
	@author		Albert Semenov
	@date		08/2008
*/

#include "Precompiled.h"
#include "BackgroundControl.h"
#include "DialogManager.h"

namespace tools
{

	BackgroundControl::BackgroundControl() :
		wraps::BaseLayout("Background.layout")
	{
	}

}
