/*!
	@file
	@author		Albert Semenov
	@date		08/2010
*/

#ifndef _867025be_2d89_4729_af93_2b681ee958be_
#define _867025be_2d89_4729_af93_2b681ee958be_

#include "SelectorControl.h"

namespace tools
{

	class AreaSelectorControlLE : public SelectorControlLE
	{
	public:
		AreaSelectorControlLE(MyGUI::Widget* _parent);
	};

}

#endif
