/*!
	@file
	@author		Albert Semenov
	@date		07/2012
*/

#ifndef _a6ef3b8c_a335_408b_8537_0e4715bdb762_
#define _a6ef3b8c_a335_408b_8537_0e4715bdb762_

namespace components
{

	class MYGUI_EXPORT_DLL IFactoryItem
	{
	public:
		virtual ~IFactoryItem() = default;
	};

}

#endif
